***************************************************************************************************
*** news_reg.do 
*** July 10, 2016
***    News Shocks in Open Economies: Evidence from Giant Oil Discoveries
***            Arezki, Ramey and Sheng (2016)
**  Quarterly Journal of Economics, 2016
*** Description:
***     This program presents summary statistics and conducts regression analysis for 
***     the dynamic effect of giant oil discoveries on macro variables.
***
*** Requires:
***     newspanel20160710.dta
***     test_predictablity.do
***     sumstat.do
***     testnlm.do
***     cumultest.do
***     graph_forQJE.do
***************************************************************************************************


version 12

clear all
drop _all
graph drop _all

set more off
set matsize 11000
capture log close


** cd to the current folder


** create the folder to save results 
global savefolder baseline
shell mkdir $savefolder

log using $savefolder\news_baseline.smcl, replace


************************************************************************************************************ 

use newspanel2016.dta,clear
disp "DateTime: $S_DATE $S_TIME"

xtset  ifscode year
xi I.year, prefix(_I)


************************************************************************************************************ 
** Part I: Summary statistics 
************************************************************************************************************ 
 

** Table 2: The Spatial Distribution of Giant Oil and Gas Discovery Events (1970-2012)							
gen decade=floor(year/10)
tostring decade, replace
replace decade=decade+"0s"

preserve
replace region_weo="Europe" if region_weo=="Central and Eastern Europe"
replace region_weo="Europe (include Central and Eastern Europe)" if region_weo=="Europe"

keep if year>1969 & year<2013
collapse (sum) discovery, by (region_weo decade)
reshape wide discovery, i(region_weo) j(decade) string
replace region_weo="World total" if region_weo==""
egen Total=rowtotal(discovery1970s-discovery2010s)
sort region_weo
foreach var of varlist discovery1970s-discovery2010s Total {
sum `var'
replace `var'=r(sum) if region_weo=="World total"
}
save $savefolder\table_ii.dta, replace
restore



** Benchmark measure of size

gen size=sizerealistic
label var size "NPV with risk adjusted discount rate and realistic production profile"

*** Figure IV: The Size Distribution of Giant Oil Discoveries: 1970-2012
hist lnurt_h if lnurt_h!=0 & year>1969 & year<2013, freq  name(lnurr,replace)
graph save $savefolder\figure_iv.gph,replace


gen lnsize=log(size)
lab var lnsize "Net Present Value of Giant Oil Discoveries in Percentage of GDP (Log)"

*** Figure V: The Distribution of Net Present Value of Giant Oil Discoveries: 1970-2012
hist lnsize if year>1969 & year<2013,freq name(lnsize,replace)
graph save $savefolder\figure_v.gph,replace




** Summary statistics of NPV measure

*** Median of size and lnsize
sum size lnsize if discovery==1 & year>1969 & year<2013, detail

*** List the maximum NPV 
sum size
list ifscode country year size if size==r(max) 


************************************************************************************************************ 
** Part II: regression analysis 
************************************************************************************************************ 

xtset ifscode year

** Generate sufficient lags for measures of discovery shocks
local discvar size discovery urr lnurt_h 

foreach var of varlist `discvar' {
  forvalues i=1/20 {
  g `var'`i'=L`i'.`var'
  }
}

** Generate lagged dependent variables

local depvar ca saving inv lngdplcu lnclcu emprate invprivgdp invpubgdp lncp_r lncg_r govspgdp lreer_ins lreer_gdp stmktcap

foreach var of varlist `depvar' {
gen `var'1=L.`var'
gen `var'2=L2.`var'
gen `var'3=L3.`var'
gen `var'4=L4.`var'
}

 

** Hypothesis testing of predictability of discoveries
** Generate Table IV

do test_predictability.do


** Sample selection 

*** BENCHMARK SETTING: all countries during 1980-2012 
drop if year<1980 | year>2012

 
** Summary statistics of key macro variables
** Generage Table B.III
do sumstat.do



** Generate country specific quadratic trends

egen tt=group(year)
gen ttsquare=tt^2

egen float dum = group(ifscode) 
sum dum
local dummax=r(max)
xi I.dum , prefix(cc_)
*g constant=1
g tt1=tt
g ttsquare1=ttsquare
  forvalues i=2/`dummax' {
  g tt`i'=tt*cc_dum_`i'
  g ttsquare`i'=ttsquare*cc_dum_`i'
  }
  



*******************************************************************************
** Dynamic panel with distributed lag model: 
** AR(p) with exogenous X and its lags (q)
** You can choose p=1 or p=2
** h-1 is the irf horitonal periods
*******************************************************************************

** Baseline setting
local p=1
local q =10
local h=20+1

local ci=90  // confidence interval

local cia=68 // alternative CI




** Key dependent variables inlcudes: ca saving inv lngdplcu lnclcu emprate invprivgdp invpubgdp lncp_r lncg_r govspgdp lreer_ins lreer_gdp stmktcap

 global keydep ca saving inv lngdplcu lnclcu emprate invprivgdp invpubgdp lncp_r lncg_r govspgdp

 
   ** Add more depedent variables
 
   * global keydep lreer_ins lreer_gdp stmktcap

   
   
******************************************************************************************************************
** Main code for Dynamic panel with distributed lag model
** It generates IRFs and P-values for hypotheses testing for all key variables
******************************************************************************************************************

capture erase $savefolder\est_coef.txt
capture erase $savefolder\est_coef.xml


foreach var of varlist $keydep {
gen start=1

if `var'==ca | `var'==saving | `var'==inv | `var'==invprivgdp | `var'==invpubgdp | `var'==govspgdp {

	** NPV with its lags: for ratio variables
	global xlist size size1-size`q' 
 
 }

else {

 ** Add control for country specific second order polynomial trend for log level variables

   global xlist size size1-size`q'  tt*

 }

 ** global dependent and independent variables.
	global dep `var'
	global lagdep `var'1-`var'`p'

 ** global yearly fixed effect
    global yearlist _Iyear_1980-_Iyear_2012

** Driscoll-Kraay Estimation

xtscc $dep $lagdep  $xlist $yearlist , fe 

** Table C.I. Estimation Results
outreg2 $lagdep  size size1-size`q'  using $savefolder\est_coef, excel  stats(coef tstat) bdec(3) cttop("xtscc") addstat(Within R-squared, e(r2_w))


** Computing IRF and hypothesis testing 

** USE nlcom and recursive macro subsitutions 

if `p'==1 { 
local rho1 "_b[`var'1]"
local irf0 "_b[size]"

forvalues i = 1/`q' {
  local i1 = `i'-1
  local irf`i' "(`rho1'*`irf`i1'' + _b[size`i'])" 		
}

local qnx=`q'+1

forvalues i = `qnx'/20 {
 local i1 = `i'-1
 local irf`i' "(`rho1'*`irf`i1'')"
}

** IRF: use the following code with a factor of 1000 otherwise it shows the error message "Maximum number of iterations exceeded."
nlcom (irf0:1000*`irf0') (irf1:1000*`irf1') (irf2:1000*`irf2') (irf3:1000*`irf3') (irf4:1000*`irf4') (irf5:1000*`irf5') (irf6:1000*`irf6') (irf7:1000*`irf7')  ///
(irf8:1000*`irf8') (irf9:1000*`irf9') (irf10:1000*`irf10') (irf11:1000*`irf11') (irf12:1000*`irf12') (irf13:1000*`irf13') (irf14:1000*`irf14') (irf15:1000*`irf15')  ///
 (irf16:1000*`irf16') (irf17:1000*`irf17') (irf18:1000*`irf18') (irf19:1000*`irf19') (irf20:1000*`irf20'), level(`ci')  post

  nlcom (irf0:_b[irf0]/1000) (irf1:_b[irf1]/1000) (irf2:_b[irf2]/1000) (irf3:_b[irf3]/1000) (irf4:_b[irf4]/1000) (irf5:_b[irf5]/1000) (irf6:_b[irf6]/1000) (irf7:_b[irf7]/1000)  ///
(irf8:_b[irf8]/1000) (irf9:_b[irf9]/1000) (irf10:_b[irf10]/1000) (irf11:_b[irf11]/1000) (irf12:_b[irf12]/1000) (irf13:_b[irf13]/1000) (irf14:_b[irf14]/1000) (irf15:_b[irf15]/1000)  ///
 (irf16:_b[irf16]/1000) (irf17:_b[irf17]/1000) (irf18:_b[irf18]/1000) (irf19:_b[irf19]/1000) (irf20:_b[irf20]/1000), level(`ci')  post


** Save all IRF in a separate data
parmest, list(,) saving($savefolder\irfnl_`var'.dta, replace) level(`ci')

** Hypothesis testing 

do testnlm.do

preserve
keep if pv1!=.
keep pv1 h0
ren pv1 pv`var'
list
save $savefolder\pv_`var'.dta, replace
restore

}

else if `p'==2 & `q'==10 {
local rho1 "_b[`var'1]"
local rho2 "_b[`var'2]"
local irf0 "_b[size]"
local irf1 "(`rho1'*`irf0' + _b[size1])"

forvalues i = 2/10 {
  local i1 = `i'-1
  local i2 = `i'-2
  local irf`i' "(`rho1'*`irf`i1'' +`rho2'*`irf`i2'' +_b[size`i'])"
}


** Generate irf and confidence interval for first 10 periods
** IRF: use the following code with a factor of 1000 otherwise it may show the error message "Maximum number of iterations exceeded."

nlcom (irf0:1000*`irf0') (irf1:1000*`irf1') (irf2:1000*`irf2') (irf3:1000*`irf3') (irf4:1000*`irf4') (irf5:1000*`irf5') (irf6:1000*`irf6') (irf7:1000*`irf7')  ///
(irf8:1000*`irf8') (irf9:1000*`irf9') (irf10:1000*`irf10') (rho1: 1000*_b[`var'1])  (rho2: 1000*_b[`var'2]) , level(`ci') post 


nlcom (irf0:_b[irf0]/1000) (irf1:_b[irf1]/1000) (irf2:_b[irf2]/1000) (irf3:_b[irf3]/1000) (irf4:_b[irf4]/1000) (irf5:_b[irf5]/1000) (irf6:_b[irf6]/1000) (irf7:_b[irf7]/1000)  ///
(irf8:_b[irf8]/1000) (irf9:_b[irf9]/1000) (irf10:_b[irf10]/1000) (rho1: _b[rho1]/1000)  (rho2: _b[rho2]/1000), level(`ci')  post

parmest, list(,) saving($savefolder\irfpa.dta, replace) level(`ci')

** The rest of IRFs 
local irf9 "_b[irf9]"
local irf10 "_b[irf10]"

forvalues i = 11/20 {
 local i1 = `i'-1
 local i2 = `i'-2
 local irf`i' "(_b[rho1]*`irf`i1'' + _b[rho2]*`irf`i2'')"
}

** Generating irf and confidence interval for 11-20 periods
** IRF: IRF: use the following code with a factor of 1000 otherwise it may show the error message "Maximum number of iterations exceeded."

nlcom (irf11:1000*`irf11') (irf12:1000*`irf12') (irf13:1000*`irf13') (irf14:1000*`irf14') (irf15:1000*`irf15')  ///
 (irf16:1000*`irf16') (irf17:1000*`irf17') (irf18:1000*`irf18') (irf19:1000*`irf19') (irf20:1000*`irf20'), level(`ci') post
 
nlcom (irf11:_b[irf11]/1000) (irf12:_b[irf12]/1000) (irf13:_b[irf13]/1000) (irf14:_b[irf14]/1000) (irf15:_b[irf15]/1000)  ///
 (irf16:_b[irf16]/1000) (irf17:_b[irf17]/1000) (irf18:_b[irf18]/1000) (irf19:_b[irf19]/1000) (irf20:_b[irf20]/1000), level(`ci')  post

parmest, list(,) saving($savefolder\irfpb.dta, replace) level(`ci')

preserve
use $savefolder\irfpa.dta, clear
append using $savefolder\irfpb.dta
keep if substr(parm,1,1)=="i"
save $savefolder\irfnl_`var'.dta, replace  // save the IRFs
restore

erase $savefolder\irfpa.dta
erase $savefolder\irfpb.dta
}

else{
di "This code only provides solutions to order `p'=1 for `q'<20, and `p'=2 and `q'=10" 
}


** Save impulse responses
preserve

use $savefolder\irfnl_`var'.dta, clear
gen n=_n-1
sort n
label var n "Year"
local tail=(100-`cia')/200
gen max`cia'=estimate+invttail(dof,`tail')*std
gen min`cia'=estimate-invttail(dof,`tail')*std

** Uncomment to plot IRF for each individual variable

* twoway rarea min`ci' max`ci' n, color(gs14) || rarea min`cia' max`cia' n,  color(gs13) || connect estimate n, color(blue) legend(off) name(irfnl_`var',replace) 
* graph save $savefolder\irfnl_`var'.gph, replace

save $savefolder\irfnl_`var'.dta, replace
restore

gen end=1
drop start-end
}



******************************************************************************************************************
** Table III. Hypothesis Tests on Cumulative Responses to an Oil News Shock
******************************************************************************************************************

do cumultest.do




******************************************************************************************************************
** Merge all IRFs of each dependent variable into one data file and
** Use graphirf to plot figures.
******************************************************************************************************************

foreach file in "lngdplcu" "ca" "saving" "inv" "lnclcu" "emprate" "invprivgdp" "invpubgdp" "lncp_r" "lncg_r" "govspgdp" {
use $savefolder\irfnl_`file'.dta, clear
ren n period
move period parm
ren estimate `file'
ren min90 `file'_min90
ren max90 `file'_max90
ren min68 `file'_min68
ren max68 `file'_max68

drop parm stderr dof t p 
save $savefolder\irf_temp_`file'.dta, replace
}

use $savefolder\irf_temp_lngdplcu.dta,clear
foreach file in "ca" "saving" "inv" "lnclcu" "emprate" "invprivgdp" "invpubgdp" "lncp_r" "lncg_r" "govspgdp"  {
merge 1:1 period using $savefolder\irf_temp_`file'.dta
drop _merge

}

sort period

ren period year
save $savefolder\irf_all.dta, replace


**Use graph_forQJE.do file to produce Figures in the paper. 


** Erase all intermediate data files.
foreach file in "lngdplcu" "ca" "saving" "inv" "lnclcu" "emprate" "invprivgdp" "invpubgdp" "lncp_r" "lncg_r" "govspgdp"  {
erase $savefolder\irf_temp_`file'.dta
erase $savefolder\irfnl_`file'.dta
erase $savefolder\pv_`file'.dta
}

erase $savefolder\pv_all.dta


log close
